/* Copyright (c) Oracle Corporation 1994.  All Rights Reserved */

/*
    This source code is provided as a debugging aid for developers
    who have purchased Oracle Objects for OLE    .  Please see the
    online help for documentation of these classes.
*/

/*
    Oracle Objects for OLE     C++ Classes
    
    This file implements OStartup, OShutdown and several internal utility functions
                           
    CREATED    ********   11/22/94
	PCHARI      MODIFIED        03/19/98
				bug#    624443  *NT 3.5.1 does not support CoInitializeEx
								and so this fix to explicitly link with ole32.dll


*/

#include "objbase.h"

#ifndef ORACL_ORACLE
#include "oracl.h"
#endif

#ifndef ORAOBJI_ORACLE
#include "oraobji.h"
#endif

#include <stdio.h>  

HINSTANCE hDLL = (HINSTANCE) NULL;               // Handle to DLL

// ----- OStartup -----------------------------------------------
// Bug 624443 : Linking explicitly to ole32.dll so that applications can run as is
// on 3.51-4.0 versions of NT/Win95. To explicitly link, we need to
// Load the ole32.dll at runtime by calling LoadLibrary( Refer : oosd.cpp, OStartup).
// Call GetProcAddress to obtain a function pointer to each exported funcion that we call.
// FreeLibrary when done (Refer : oosd.cpp, OShutdown)

// As we are calling ole32.dll exported functions
// through this pointer, the compiler does not generate external references, so there
// is no need to link with an import library.

//#ifndef _MT   
//oboolean OStartup(void)
//#else

int gThreadingModel = OSTARTUP_MULTITHREADED;
CRITICAL_SECTION gCritSect;

oboolean OStartup(int ThreadingModel)
//#endif
{
#ifndef _MT
	oglobal *ogp = ssoo4wGetGlobal();  // probably allocates memory
    if (ogp->ogtried)
        return(FALSE);  // have already tried to startup
    
    // initialize OLE
    ogp->ogtried = TRUE;
#endif

	// Bug 624443 : call LoadLibrary to explicitly link with ole32.dll
	HRESULT hr;
	LPVOID  CoFuncPtr;
		
	hDLL = LoadLibrary(CURR_MSOLE_DLL);
	if (hDLL != NULL)
	{
		CoFuncPtr = OOGetProcAddress(COINITIALIZEEX); 
		if (!CoFuncPtr)   
		{
			// call the function OleInitialize   
			CoFuncPtr = OOGetProcAddress(OLEINITIALIZE); 
			if ( !CoFuncPtr)	return(FALSE);
			hr = ((OLEINITDLLFUNC)CoFuncPtr)(0);
			
		}   
		else 
		{ 
			InitializeCriticalSection (&gCritSect);
			EnterCriticalSection(&gCritSect);

			// call the function
			if (ThreadingModel == OSTARTUP_MULTITHREADED)
			{
				gThreadingModel = OSTARTUP_MULTITHREADED;
				// call the function - CoInitializeEx
				hr = ((COINITDLLFUNC)CoFuncPtr)(0, COINIT_MULTITHREADED);
			}
			else
			{
				// call the function OleInitialize   
				gThreadingModel = OSTARTUP_APARTMENTTHREADED;
				CoFuncPtr = OOGetProcAddress(OLEINITIALIZE); 
				if ( !CoFuncPtr)	return(FALSE);
				hr = ((OLEINITDLLFUNC)CoFuncPtr)(0);
			}
			LeaveCriticalSection(&gCritSect);

		}
	}
	else	return FALSE;
	
    SCODE sc = GetScode(hr);
    if (sc == S_OK)
    {
#ifndef _MT
        ogp->ogdid = 1;  // we initialized 
#endif
        return(TRUE);
    }
    else if (sc == S_FALSE)   
    {
#ifndef _MT
        ogp->ogdid = 2;  // we didn't, but its running
#endif
        return(TRUE);
    }
    else
    {
#ifndef _MT
        ogp->ogdid = 0;  // no ole (some error)  
#endif		
        return(FALSE);
    }
}

// ----- OShutdown -----------------------------------------------

void OShutdown(void)
{
	// Bug 624443
	LPVOID  CoFuncPtr;

#ifndef _MT
    oglobal *ogp = ssoo4wGetGlobal();
    
    // shutdown ole, if needed
    if (ogp->ogtried && ogp->ogdid == 1)
#endif
    {	
		// we initialized, we should uninitialize
		// Bug 624443 : call the function CoUnInitialize   
		CoFuncPtr = OOGetProcAddress(COUNINITIALIZE); 
		if (CoFuncPtr)   
			((COUNINITDLLFUNC)CoFuncPtr)();
		FreeLibrary(hDLL);
		hDLL = (HINSTANCE) NULL;
	}
#ifndef _MT    
    // go ahead and free the global state
    ssoo4wfree();
#endif
    
    return;
}

// ----- Check for Initialization -----------------------------------------------

// check to make sure library is initialized 
/*
    We only need to check whether OLE is initialized from the creation
    of an OSession.  Because if that doesn't result in an open OSession,
    nothing else will get to the point of calling OLE.
*/

oboolean CheckOLE(void)
{
#ifdef _MT
	return TRUE;
#else
    oglobal *ogp = ssoo4wGetGlobal();
    
    if (ogp->ogtried && ogp->ogdid)
        return(TRUE);
    else
        return(FALSE);
#endif
}


// ----- String Utilities -----------------------------------------------

// for the Windows version this needs to use OLE routines 

char *OObjectAllocString(const char *instr)
{
#ifdef WIN32
    return((char *) SysAllocStringA(instr));
#else
   return((char *) SysAllocString(instr));
#endif
}

void OObjectFreeString(char *instr)
{
    if(instr)
       SysFreeString((BSTR) instr);
}

char *OObjectAllocStringLen(char *instr, unsigned int slen)
{
    // allocate string by length (takes care of embedded nulls) 

#ifdef  WIN32
        return((char *) SysAllocStringLenA(instr, slen));
#else
        return((char *) SysAllocStringLen(instr, slen));
#endif
}

unsigned int OObjectStringLen(char *instr)
{
#ifdef WIN32
    return(SysStringByteLen((BSTR) instr));
#else
    return(SysStringLen((BSTR) instr));
#endif
}

